from __future__ import annotations

import typing as t


import requests as r
from .models import Anime, Character


class AniClient(r.Session):
    def __init__(self, token: t.Optional[str] = None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.token = token

    def anime(self: AniClient, name: str) -> Anime:
        return Anime(self, name)

    def character(self: AniClient, name: str):
        return Character(self, name)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, trace):
        ...



ANIME_QUERY = """
query ($search: String) {
  Media (search: $search, type: ANIME) {
    title {
      romaji
      english
      native
    }
    description
    averageScore
    status
    tags{
      name
    }
  }
}
"""

CHARACTER_QUERY = """
query ($search: String) {
  Character(search: $search) {
    id
    name {
      first
      last
    }
    description
    image {
      large
    }
  }
}
"""

BASE = "https://graphql.anilist.co"



from __future__ import annotations


import typing as t
from ratelimit import limits, sleep_and_retry
from .constants import ANIME_QUERY, CHARACTER_QUERY, BASE


if t.TYPE_CHECKING:
    from .client import AniClient

from .errors import HTTPException


class QueryHandler(type):
    def __call__(cls, *args, **kwargs):
        self = type.__call__(cls, *args, **kwargs)
        resp = self._build_query()
        return self


class BaseEndpoint(object, metaclass=QueryHandler):
    def __init__(self, client: AniClient, name: str):
        self.data = {}
        self.client = client
        self._name = name

        self.QUERY = NotImplemented

    @sleep_and_retry
    @limits(calls=1.5, period=1)
    def _build_query(self):
        payload = {
            "query": self.QUERY,
            "variables": {"search": self._name},
        }
        resp = self.client.post(BASE, json=payload)
        if resp.status_code != 200:
            print(resp.text)
            raise HTTPException(
                f"Status code: {resp.status_code} response: {resp.text}"
            )
        returned = resp.json()
        self.data = returned["data"]

    def __repr__(self):
        return f"<name={self._name}, type={list(self.data.keys())[0]}>"

    def __iter__(self):
        for key, value in self.__dict__.items():
            yield key, value


class Anime(BaseEndpoint):
    def __init__(self, client: AniClient, name: str):
        super().__init__(client, name)
        self.QUERY = ANIME_QUERY
        self.client = client

    @property
    def id(self):
        return self.data["Media"]["id"]

    @property
    def title(self):
        return self.data["Media"]["title"]

    @property
    def description(self):
        return self.data["Media"]["description"]

    @property
    def averageScore(self):
        return self.data["Media"]["averageScore"]

    @property
    def status(self):
        return self.data["Media"]["status"]

    @property
    def tags(self):
        return [tag["name"] for tag in self.data["Media"]["tags"]]


class Character(BaseEndpoint):
    def __init__(self, client: AniClient, name: str):
        super().__init__(client, name)
        self.QUERY = CHARACTER_QUERY
        self.client = client

    @property
    def name(self):
        return self.data["Character"]["name"]

    @property
    def description(self):
        return self.data["Character"]["description"]

    @property
    def image(self):
        return self.data["Character"]["image"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nino']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0',
 'flake8>=3.9.2,<4.0.0',
 'ratelimit>=2.2.1,<3.0.0',
 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'nino',
    'version': '0.1.2',
    'description': 'A small unofficial AniList API wrapper made for python',
    'long_description': '# Basic usage\n\nCharacter\n```py\nfrom nino import AniClient\n\nwith AniClient() as client:\n    character = client.character("Momonga")\n    print(character.name)\n\n```\n\nAnime\n```py\nfrom nino import AniClient\n\nwith AniClient() as client:\n    anime = client.anime("Overlord")\n    print(anime.title)\n\n```',
    'author': 'Andy',
    'author_email': 'yrisxl@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/an-dyy/anilist-wrapper',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
